 /********************************************************************************************************************************************************
 *                                                                                                                                                       *
 *  Project:         FFT Spectrum Analyzer                                                                                                               *
 *  Target Platform: ESP32                                                                                                                               *
 *                                                                                                                                                       * 
 *  Version: 1.0                                                                                                                                         *
 *  Hardware setup: See github                                                                                                                           *
 *  Spectrum analyses done with analog chips MSGEQ7                                                                                                      *
 *                                                                                                                                                       * 
 *  Mark Donners                                                                                                                                         *
 *  The Electronic Engineer                                                                                                                              *
 *  Website:   www.theelectronicengineer.nl                                                                                                              *
 *  facebook:  https://www.facebook.com/TheelectronicEngineer                                                                                            *
 *  youtube:   https://www.youtube.com/channel/UCm5wy-2RoXGjG2F9wpDFF3w                                                                                  *
 *  github:    https://github.com/donnersm                                                                                                               *
 *                                                                                                                                                       *  
 ********************************************************************************************************************************************************/
 

//This file is only to display the logo, If you don't like the logo comment out:  drawLogo(); around line 81 in the main sketch
//You can also change it. It's a jpg converted to RGB 46x50px
// Image Size     : 46x50 pixels
// Memory usage   : 4600 bytes


#if defined(__AVR__)
    #include <avr/pgmspace.h>
#elif defined(__PIC32MX__)
    #define PROGMEM
#elif defined(__arm__)
    #define PROGMEM
#endif


// 'logo RGB', 46x50px
const unsigned long logo [] PROGMEM = {
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffd42a, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffae0e, 0x00faad09, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00fbac0a, 0x00faa708, 0x00000000, 0x00000000, 0x00000000, 0x00faab0a, 0x00faa908, 0x00000000, 0x00000000, 0x00000000, 0x00fba607, 0x00faaa08, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00feb917, 0x00ffff55, 0x00000000, 0x00000000, 0x00000000, 0x00faae0c, 0x00fab30b, 0x00ffaa08, 0x00000000, 0x00000000, 0x00fabd0c, 0x00fac70d, 0x00ffff7f, 0x00000000, 0x00000000, 0x00fab00a, 0x00fab30d, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffb40a, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00feb917, 0x00faaa0a, 0x00faa708, 0x00000000, 0x00000000, 0x00ffaa0b, 0x00fadf11, 0x00fabd04, 0x00000000, 0x00000000, 0x00fad013, 0x00fffa1e, 0x00ffa707, 0x00000000, 0x00fba600, 0x00fade0d, 0x00fbbe11, 0x00000000, 0x00000000, 0x00000000, 0x00faa708, 0x00fbb00b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00fab60e, 0x00fac10d, 0x00ffa108, 0x00000000, 0x00faa404, 0x00faed37, 0x00faee2b, 0x00fe9c00, 0x00000000, 0x00ffef3c, 0x00dee958, 0x00fca900, 0x00000000, 0x00fed612, 0x00ffff45, 0x00ffbe07, 0x00000000, 0x00000000, 0x00fab808, 0x00fac211, 0x00ffff7f, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffaf0f, 0x00ffbf15, 0x00000000, 0x00000000, 0x00000000, 0x00fbad09, 0x00faf124, 0x00facc04, 0x00ffbf00, 0x00ffaf00, 0x00f9e35c, 0x00ffffa7, 0x00fec800, 0x00e78f00, 0x00bcb85f, 0x008d9982, 0x00c7a01c, 0x00c5851c, 0x00d4d069, 0x00b7bf75, 0x00d88e00, 0x00000000, 0x00feb900, 0x00fdf926, 0x00fac611, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00feac10, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00feae0d, 0x00fab00a, 0x00faa909, 0x00000000, 0x00000000, 0x00fe9e00, 0x00fadf36, 0x00fbff7a, 0x00fbbf00, 0x00fa9100, 0x00feee52, 0x00f5f9da, 0x00c5bd4c, 0x00c2ae88, 0x00dadff1, 0x00f7f6ff, 0x00e4ecf8, 0x00dae7fc, 0x00d7dcf2, 0x0092929b, 0x00a3844c, 0x00c7941c, 0x00d7d548, 0x00f2f33f, 0x00ffbd00, 0x00000000, 0x00000000, 0x00faa609, 0x00faaf0b, 0x00fbab0b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00fca80a, 0x00fad317, 0x00fac304, 0x00ff9b00, 0x00000000, 0x00fbc70a, 0x00fbffcf, 0x00faff95, 0x00ffcb00, 0x00f5dd22, 0x00bfc8c1, 0x00d8deff, 0x00ffffff, 0x00ffffff, 0x00f3f3f5, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00f7ffff, 0x00dfe9fa, 0x00ccd9e6, 0x00cec083, 0x0090660c, 0x00000000, 0x00ffc500, 0x00f9d717, 0x00fab90e, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00fac90e, 0x00faff53, 0x00fad62a, 0x00fa9f00, 0x00faa800, 0x00fdfc8f, 0x00ffffff, 0x00f9fd5b, 0x00b6ac38, 0x00e5e7fb, 0x00ffffff, 0x00f3f3f3, 0x00a9aaae, 0x00e4e4e6, 0x00ffffff, 0x00fbfbfb, 0x00fdfdfd, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x008c9ad5, 0x00846a39, 0x00dee82f, 0x00fdf423, 0x00fea408, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00fead09, 0x00ffac0f, 0x00000000, 0x00000000, 0x00000000, 0x00fa9e00, 0x00fae94f, 0x00fbffda, 0x00fbf166, 0x00fcc700, 0x00f8e83f, 0x00e7e8d7, 0x00bbbba5, 0x00c8cbdf, 0x00ffffff, 0x00f4f4f4, 0x00939599, 0x00b8b9bd, 0x00ffffff, 0x00c4c5c7, 0x00eaebec, 0x00ffffff, 0x00ffffff, 0x00fdfdfd, 0x00ececec, 0x00b5b6c7, 0x00949169, 0x00e4e174, 0x00d0dde3, 0x00d8dd74, 0x00fcbd05, 0x00000000, 0x00000000, 0x00000000, 0x00ffff7f, 0x00ffaf0b, 0x00ffff55, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffaf0f, 0x00faac0a, 0x00fabc0f, 0x00fcab05, 0x00fea900, 0x00000000, 0x00fcb800, 0x00fbffa7, 0x00fbffff, 0x00ffff97, 0x00f4ed55, 0x00868565, 0x00aeadac, 0x00ffffff, 0x00ffffff, 0x00bbbcc0, 0x009c9da0, 0x00e8eaee, 0x00b2b7d2, 0x00b8bbc9, 0x00ffffff, 0x00d7d8db, 0x00f0f0f0, 0x00ffffff, 0x00fefefe, 0x00dadae7, 0x00a2a07b, 0x00bfbe85, 0x00ffffba, 0x00fef61d, 0x00ffff00, 0x00ffaa00, 0x00fba300, 0x00fbbb10, 0x00fab50c, 0x00fbaa09, 0x00ffff7f, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffa405, 0x00facd12, 0x00fbf434, 0x00fbd836, 0x00fcc107, 0x00fca700, 0x00fbde2b, 0x00fbffda, 0x00fffffe, 0x00ecebc8, 0x00b0b0ab, 0x00f2f2f7, 0x00efefed, 0x00aeaeb0, 0x0077777b, 0x009a9ca2, 0x009599a4, 0x00837c53, 0x00837a55, 0x00575759, 0x00c3c6cf, 0x00f8f9f9, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00d0d1f2, 0x00a7ab76, 0x00c08b00, 0x00f6ae00, 0x00fdcf1f, 0x00faed3d, 0x00fae519, 0x00fbab09, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffa309, 0x00fbe534, 0x00fbffb3, 0x00fcffbc, 0x00fced75, 0x00fbe83f, 0x00fdfa9b, 0x00fffff0, 0x00b4b1a4, 0x00efeffa, 0x00cacacc, 0x0077767a, 0x008a8b8d, 0x009a9ca4, 0x00595e80, 0x00978d51, 0x00e0c322, 0x0047402e, 0x002c281b, 0x00827b68, 0x00b7bac8, 0x00bebfc3, 0x00f0f0f0, 0x00ffffff, 0x00fdfdfd, 0x00ffffff, 0x00ffffff, 0x00beb9cc, 0x0084895a, 0x00d7eea5, 0x00feff55, 0x00fbab00, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffa712, 0x00fbe75b, 0x00fcfff0, 0x00fcffff, 0x00fcffdb, 0x00fefcc7, 0x00ffffd9, 0x00cbcac5, 0x00d9d9e1, 0x005e5c67, 0x0094949b, 0x00c1c3ca, 0x00a7aaac, 0x007e7664, 0x00f9da5c, 0x00b5a135, 0x00868db3, 0x00767772, 0x004e3807, 0x004b3c35, 0x00898e96, 0x00e0e0e4, 0x00ffffff, 0x00fefefe, 0x00fdfdfd, 0x00c1c0c0, 0x009ea1aa, 0x00bdc5e1, 0x00f2f465, 0x00fbae00, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffae0c, 0x00fbad09, 0x00fab50e, 0x00fcaf0b, 0x00fbab03, 0x00ffa80f, 0x00fcb625, 0x00fcef84, 0x00fcffe7, 0x00fcfcf0, 0x00ffffee, 0x00f5f5d7, 0x00c5bfd1, 0x00a9a7bd, 0x0095939f, 0x00a8a8ad, 0x00c0c3d2, 0x007e7d73, 0x00857003, 0x00fdde38, 0x00998837, 0x009fa4c2, 0x0078797f, 0x004d451c, 0x0060615b, 0x007c7e84, 0x00a2a2a2, 0x00e6e5e4, 0x00ffffff, 0x00ffffff, 0x00c4c4cd, 0x0070746a, 0x00fefb4d, 0x00fed900, 0x00ff9c00, 0x00fba600, 0x00fbaa03, 0x00fcb310, 0x00fbb00b, 0x00fbac09, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffaa0a, 0x00faa606, 0x00fbc811, 0x00fbf62a, 0x00fbf16f, 0x00fbea7c, 0x00fbe578, 0x00fdf0a8, 0x00fcfcde, 0x00f4f7fa, 0x00fefdf1, 0x00d4c5d7, 0x009a8bb8, 0x00928faa, 0x00a09eb0, 0x00b1b1b8, 0x00c6c8cb, 0x009698a8, 0x00605a32, 0x00f5da22, 0x00c1a931, 0x00878cab, 0x008a8e93, 0x005d531a, 0x008c8581, 0x0085888e, 0x006f7072, 0x0088888b, 0x00afafaf, 0x00f0f0ef, 0x00ffffff, 0x00daded7, 0x00c8bd33, 0x00bd9e15, 0x00dfca46, 0x00fdf070, 0x00faf736, 0x00fada12, 0x00faac09, 0x00faaa08, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffa809, 0x00fbca17, 0x00fbfe79, 0x00fbfff2, 0x00ffffff, 0x00ffffee, 0x00f8f7f2, 0x00f6f7ff, 0x00fbf7fe, 0x00e3d1e1, 0x00c3b6d7, 0x00ada8b8, 0x009f9bac, 0x008b8a99, 0x00a1a1a5, 0x00d4d9e4, 0x00686f8f, 0x0099820d, 0x00ffea1a, 0x0086895e, 0x005a6355, 0x00b98d46, 0x00df9263, 0x00726760, 0x00818a94, 0x006e6f72, 0x00606063, 0x00868687, 0x00cececd, 0x00ffffff, 0x00cdd0ef, 0x007f89b5, 0x00ddeee0, 0x00ffff9c, 0x00fadb1e, 0x00faab02, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00fcac08, 0x00fee144, 0x00e6edd7, 0x00d8e1fe, 0x00efeeff, 0x00ffffff, 0x00fdfaff, 0x00fffaf1, 0x00dbd2ec, 0x00988faf, 0x00696077, 0x0095949f, 0x007b7e87, 0x00737f8a, 0x00937b77, 0x008f7e6e, 0x00b1a852, 0x00946e1a, 0x005c4227, 0x00e8a87d, 0x00ffbf80, 0x00ab9281, 0x00a5b4c3, 0x009fa1a5, 0x00909295, 0x00717174, 0x00565557, 0x0085868b, 0x008b8ead, 0x00d2d592, 0x00fef74a, 0x00fbb000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffbf3f, 0x00ffa706, 0x00fbac00, 0x00fcc11b, 0x00f4ea82, 0x00dce9f0, 0x00dbd7ff, 0x00eef3ff, 0x00eceeff, 0x00fcebe9, 0x00ece1c9, 0x00a29ac1, 0x007a7999, 0x00757e94, 0x005a585e, 0x00906d54, 0x00ffd0a5, 0x00a5a2ab, 0x005d4d5f, 0x006a4f49, 0x002a211d, 0x00c38e68, 0x00ffca8c, 0x00be916f, 0x0074818c, 0x00838d97, 0x00afb5be, 0x00a9a9ac, 0x00818186, 0x003c3d51, 0x00888a51, 0x00d6d91f, 0x00efb800, 0x00fca600, 0x00ffa400, 0x00fe9d0c, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffaa0e, 0x00fba808, 0x00faba0d, 0x00fbdc13, 0x00fbe944, 0x00fbf38f, 0x00fcfcbd, 0x00ffffbf, 0x00e6ebe6, 0x00c9d8fd, 0x00ccb9f6, 0x00ccb6f9, 0x00cbc6e7, 0x00e7d49b, 0x00cfc168, 0x005b5062, 0x006b555f, 0x00645752, 0x00b5825a, 0x00a4846d, 0x004e5357, 0x00d9dde0, 0x008ba6bb, 0x00080505, 0x00dd9e70, 0x00ffdc9e, 0x00edaf80, 0x00302b27, 0x006b4d36, 0x008b7362, 0x009ea5ad, 0x00bbbdc3, 0x008c8d91, 0x00959587, 0x00868677, 0x00f2f197, 0x00fffd88, 0x00faeb58, 0x00fae11b, 0x00fac80e, 0x00faab0a, 0x00fbaa0b, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffb40a, 0x00faa809, 0x00fbb80d, 0x00fbda0d, 0x00fbf652, 0x00fbffc0, 0x00ffffdd, 0x00fffee4, 0x00c8d5f9, 0x0091b1fe, 0x009d73cc, 0x009257bb, 0x0099abf2, 0x00d5d3d9, 0x00d9d69d, 0x00a59165, 0x00986f65, 0x004d4b4f, 0x004e392d, 0x0080593c, 0x00655347, 0x00a7abaf, 0x00aca39e, 0x00e1aa7a, 0x00ffbd86, 0x00b07f5c, 0x00cf9a6f, 0x00ac7a55, 0x00d48950, 0x009e795e, 0x00929fac, 0x00aaabaf, 0x00c7c9ca, 0x008789a8, 0x00b8b6bc, 0x00ffffff, 0x00fbffc1, 0x00fbfe69, 0x00fae519, 0x00fac40b, 0x00fbaa09, 0x00ffaa0a, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffaa15, 0x00fbaa00, 0x00ffbc00, 0x00edcd47, 0x00a7bcf5, 0x00baccf6, 0x008d9ff7, 0x005e53ba, 0x00895e99, 0x005f67d3, 0x00a6bbff, 0x00a9bfff, 0x00f8f8d0, 0x00c5d0ba, 0x003e4859, 0x00764c2f, 0x00ffcf95, 0x00f0ac79, 0x00a36c43, 0x00e2965b, 0x00fab578, 0x00b37f58, 0x00443226, 0x006f513c, 0x00c68d63, 0x008b6f59, 0x008a9098, 0x00b8bbc0, 0x00b7b8c0, 0x007c7f9a, 0x006c6d5a, 0x00f1f057, 0x00fede26, 0x00fabc00, 0x00fcac00, 0x00ff9e06, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffa200, 0x00e0ba40, 0x00b8cded, 0x00ccd7ea, 0x007c8edb, 0x003c337a, 0x00b795a4, 0x003b52ce, 0x005b6dcf, 0x00a0b5ff, 0x00e3f2f5, 0x00d7c28c, 0x0077513e, 0x00b2805d, 0x00f3b584, 0x00cf9366, 0x00e7a672, 0x00d0a786, 0x00897160, 0x00675950, 0x008d7360, 0x00f1a367, 0x006b482d, 0x00647586, 0x00c4c9d1, 0x00a0a0a6, 0x006f6f70, 0x00949386, 0x00e3e3b2, 0x00ffff73, 0x00fad704, 0x00faa500, 0x00ffaa00, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffaa55, 0x00faad04, 0x00facf19, 0x00fbf774, 0x00ffffc8, 0x00ffffe3, 0x009eb2ff, 0x006681ea, 0x004d5ca7, 0x00977da2, 0x0058549d, 0x005f7bec, 0x00bed4ff, 0x00ffffdd, 0x00c5a170, 0x00997059, 0x00edc59f, 0x00e69c63, 0x005a3f2b, 0x007d8489, 0x00b8c2cc, 0x00f8ffff, 0x009da9b2, 0x00322418, 0x00cc9a71, 0x00a5978b, 0x009ea3a8, 0x009b9c9e, 0x00686971, 0x005d5b43, 0x00eeedb7, 0x00ffffe8, 0x00faf9ff, 0x00fbffeb, 0x00fbff95, 0x00fade32, 0x00fab606, 0x00ffa707, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffbf1f, 0x00faa906, 0x00fab40a, 0x00fae10e, 0x00faff3d, 0x00faff98, 0x00ffff9c, 0x00d5dbc2, 0x00c0ccdc, 0x007086e4, 0x00325dff, 0x003c4fcf, 0x003b44bb, 0x001131bb, 0x006e73af, 0x00f7f9d8, 0x0089805c, 0x00d4a588, 0x00ffd492, 0x00a97a55, 0x0090959a, 0x00bac5d1, 0x00a3a5aa, 0x00acabac, 0x00c2c2bf, 0x00a7a6a6, 0x00b8c2cc, 0x00c8d0d8, 0x00d8d7d8, 0x00dededf, 0x00b1b1b1, 0x00d4d4d0, 0x00a7a7ac, 0x00a9a657, 0x00ffffa3, 0x00fbffa4, 0x00fbffa6, 0x00faff5f, 0x00faf017, 0x00fac10d, 0x00faa608, 0x00feac10, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffad09, 0x00ffb40d, 0x00ffb611, 0x00ffb302, 0x00ffbb00, 0x00ffb800, 0x00faed0d, 0x00bac44d, 0x0025209c, 0x006b5c95, 0x007e80b3, 0x00696697, 0x0045353f, 0x008c592f, 0x00ece5aa, 0x00b6af90, 0x00c68a63, 0x00a37453, 0x009299a2, 0x00aab4bf, 0x00737274, 0x006c757c, 0x00c0c6cc, 0x00c2c1c3, 0x00c9cbd0, 0x00898b90, 0x00818184, 0x00868689, 0x009e9fa0, 0x00fefeff, 0x00fafbff, 0x00b8b7b3, 0x00f1f06f, 0x00fce200, 0x00faaf00, 0x00fbb400, 0x00fbb200, 0x00fab10e, 0x00fcb40c, 0x00fcac0a, 0x00feaf11, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00feae28, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffb505, 0x00fefe3d, 0x00d3b477, 0x00d89167, 0x00ffc26d, 0x00d3934f, 0x00bb9d70, 0x00e5c290, 0x009f6641, 0x00c49e77, 0x00e9f0be, 0x00788088, 0x007f899a, 0x007e8388, 0x00555c62, 0x004e555d, 0x007a6656, 0x00ccbdb2, 0x00bbc1c9, 0x00717275, 0x00bbbaba, 0x00f4f4f3, 0x00f1f1f0, 0x00aeaeae, 0x00919194, 0x00b1b0a5, 0x00e6e4c9, 0x00fffffd, 0x00faf581, 0x00fabe00, 0x00ff9900, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00eb8b1d, 0x0049250c, 0x0034110d, 0x003d1d0d, 0x0046230d, 0x0043200e, 0x00421e0c, 0x003c250a, 0x0036181b, 0x006a3b1a, 0x00e2a052, 0x007b5e4b, 0x00846e5b, 0x00ebc4a1, 0x00ffc58f, 0x00dfa275, 0x0084543b, 0x00bfba91, 0x00d8dcbe, 0x0071747a, 0x004a4849, 0x0071655c, 0x00c5b09e, 0x00ffc183, 0x00ce9162, 0x008c959d, 0x00acaeb1, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00a8a8b5, 0x00b9b570, 0x00ffffb6, 0x00faffcb, 0x00fbffff, 0x00fbffb2, 0x00fad521, 0x00fb9e00, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00ffbf3f, 0x0073420f, 0x00373600, 0x008e9500, 0x00635300, 0x00191600, 0x003c3500, 0x00363100, 0x00483800, 0x00928b00, 0x00221700, 0x008a522b, 0x00d7a77e, 0x00f8bd8b, 0x00ad8465, 0x00d0ab8d, 0x00d69e74, 0x00be7d57, 0x00b19c76, 0x00dce2b7, 0x008d8f96, 0x00828a9a, 0x00a0724d, 0x00ffdc96, 0x00e8a16a, 0x009f9187, 0x00848a91, 0x00c7c7c7, 0x00cececf, 0x00dfdfdf, 0x00fbfbfb, 0x00ffffff, 0x00ffffff, 0x00cacba6, 0x00faf018, 0x00fbd112, 0x00fae041, 0x00faf37d, 0x00faff4e, 0x00fae113, 0x00faac0a, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00ffb233, 0x0044220d, 0x00836d00, 0x00eeca00, 0x00816200, 0x00bf9700, 0x00bf9100, 0x00be9100, 0x00f3c100, 0x00b39900, 0x00321600, 0x00eb9f60, 0x00e5b48b, 0x008d684e, 0x00b99e85, 0x00e7bc93, 0x00bb8863, 0x00f5af7d, 0x00d29e70, 0x00949b85, 0x00c6c7c2, 0x007a8487, 0x00836e63, 0x00af7d51, 0x00978a82, 0x0093a0ae, 0x008d8c8c, 0x00b5b4b4, 0x00c2c2c3, 0x00ececed, 0x00ececeb, 0x00b2b2b1, 0x00ffffff, 0x00d7daf4, 0x00d8dc6f, 0x00fec400, 0x00fb9200, 0x00faac00, 0x00fbb803, 0x00fbcb15, 0x00fab70c, 0x00fca807, 0x00ffbf3f, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00743c14, 0x00806800, 0x004c3400, 0x003b2d00, 0x00fffa00, 0x00b58a00, 0x008b6a00, 0x00cdbf00, 0x00291b00, 0x00765636, 0x00dab681, 0x00986a4e, 0x00e3b283, 0x00bf936f, 0x00caa488, 0x00bd8e6b, 0x00f2ab7a, 0x00ca986e, 0x00e8edb3, 0x00d7d6ad, 0x00b0afa6, 0x00848993, 0x00474f5a, 0x00b5bec6, 0x00979699, 0x00a2a1a0, 0x00cececf, 0x00d4d4d4, 0x00d3d3d5, 0x00dedfe0, 0x00c2c1bf, 0x00979696, 0x00c5c5c7, 0x00d1dde2, 0x00ffff82, 0x00faaa00, 0x00000000, 0x00000000, 0x00000000, 0x00ffbf3f, 0x00ffb10b, 0x00ffbf3f, 0x00000000, 
  0x00000000, 0x00000000, 0x00ffb624, 0x007c4d0d, 0x00be9900, 0x00332700, 0x00b08600, 0x00dfb400, 0x00ab8200, 0x00d19f00, 0x00f4e000, 0x00482a00, 0x00bea952, 0x00d9cb84, 0x00e9a688, 0x009d775c, 0x00a58c7d, 0x00ffe6bc, 0x00d89e77, 0x00d0926d, 0x00b99986, 0x00aec2b3, 0x00dbe5d5, 0x00f3feff, 0x008b97a6, 0x008b98a9, 0x008c94a0, 0x00727b86, 0x00bbbcc2, 0x00fdffff, 0x00ffffff, 0x00e9f4ff, 0x00b2bbc7, 0x00e5eefc, 0x00c0cde6, 0x00656c6f, 0x00e8df65, 0x00ffffb6, 0x00fffe35, 0x00ffb900, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00ff9621, 0x003d2901, 0x00332a00, 0x00000000, 0x00423300, 0x003f2b00, 0x00342700, 0x005e4800, 0x008e8000, 0x003a2101, 0x00be860c, 0x00ca9318, 0x00804d2f, 0x00c38447, 0x00ffbd71, 0x00bc7337, 0x00eba15e, 0x00ffb161, 0x00b38a63, 0x00b2967c, 0x00b08f69, 0x00a28265, 0x00805d38, 0x00997a5d, 0x008f6845, 0x00bb9e7c, 0x00ffffff, 0x00fff3d4, 0x00c79e7f, 0x00d7b28e, 0x00ecdece, 0x00ac9784, 0x00d6b797, 0x00a47d58, 0x00c97f00, 0x00dfb617, 0x00f2d538, 0x00db990e, 0x00f0871b, 0x00ffa127, 0x00c06718, 0x00c76d1b, 0x00fea930, 0x00000000, 
  0x00000000, 0x00000000, 0x00975a11, 0x00000001, 0x00000005, 0x000c0504, 0x00000004, 0x00000000, 0x00000005, 0x00000002, 0x00000000, 0x00000001, 0x00351b08, 0x002c0c00, 0x003b2200, 0x00573209, 0x002e0f00, 0x00271b00, 0x002f1500, 0x006f450f, 0x002e1200, 0x00311200, 0x002d0900, 0x00351400, 0x00341600, 0x002d0a00, 0x00371700, 0x004b2808, 0x00cda35c, 0x003f1d00, 0x00270900, 0x001b0000, 0x009a601f, 0x00864f11, 0x00290700, 0x00532a00, 0x003a180c, 0x00290e00, 0x00230600, 0x00220b09, 0x003c250a, 0x00351a05, 0x00241807, 0x001b0d05, 0x00713f0e, 0x00feb12c, 
  0x00000000, 0x00f99622, 0x00472a0a, 0x00cb8f31, 0x00ffd041, 0x00ffea47, 0x008d641d, 0x00614014, 0x00ffce3d, 0x001a1105, 0x00000000, 0x009f6a20, 0x00f5d23e, 0x00ffd23d, 0x00e7af32, 0x00000000, 0x00896c1e, 0x00ffe444, 0x00ac7c24, 0x00020300, 0x00cfa129, 0x00e2b531, 0x00fdcf36, 0x00d1a028, 0x006b4c14, 0x00eec734, 0x00e9cd37, 0x007e6014, 0x00000000, 0x0075570e, 0x00ffef3d, 0x00d6b62f, 0x00000000, 0x002f1d00, 0x00dfda38, 0x001b1705, 0x002b2409, 0x00e9c82f, 0x002d2109, 0x00e1c42d, 0x0046390e, 0x00272208, 0x00fff335, 0x00e5d02e, 0x00322a09, 0x00be6916, 
  0x00000000, 0x00b05d12, 0x00654c14, 0x00ffe23e, 0x00ac7724, 0x00bca12c, 0x0047330d, 0x00d4a92a, 0x00fac431, 0x00000000, 0x00060301, 0x00fed633, 0x00d7ad2d, 0x00ab9226, 0x007c6017, 0x00725514, 0x00fff639, 0x00ffdd30, 0x00ffff3a, 0x00503b0d, 0x0097761c, 0x00fff537, 0x00e7cc2d, 0x00735814, 0x00c9a322, 0x00f3d72f, 0x00fad72e, 0x00f1cb29, 0x003e320a, 0x00fff531, 0x00edcb28, 0x00ffff34, 0x0040330a, 0x005d510f, 0x00ffff3c, 0x00816a14, 0x00c2a51e, 0x00d5b822, 0x0061510f, 0x00ffff2e, 0x0058480c, 0x00f8de25, 0x00fff229, 0x00ffff34, 0x00958a16, 0x007f370d, 
  0x00000000, 0x0071370c, 0x008a7d1a, 0x00ffff31, 0x008c8e17, 0x002b2203, 0x00000000, 0x00ffff35, 0x00816e13, 0x00000000, 0x004e410c, 0x00ffff32, 0x00beb01c, 0x0070610d, 0x00000000, 0x00e1c822, 0x00ecd222, 0x00624f0d, 0x00a2991b, 0x00000000, 0x002c2c05, 0x00ffff2f, 0x00150f01, 0x00000000, 0x00fffb26, 0x00d7c31c, 0x00f2e923, 0x00998815, 0x008a7d14, 0x00fff224, 0x009b8713, 0x00ffff26, 0x00483a0a, 0x00bcad19, 0x00ffff26, 0x00ddc31d, 0x00ffff26, 0x00655a0d, 0x00a79615, 0x00d1bc19, 0x006b5d0d, 0x00ffff27, 0x0068550b, 0x009e9414, 0x003e3a08, 0x00c26918, 
  0x00ff9f2a, 0x0058360b, 0x00c8c51c, 0x00f3f724, 0x00ffff2c, 0x00352d08, 0x004a450a, 0x00ffff2a, 0x00121201, 0x00000000, 0x00a39316, 0x00feff26, 0x00feff27, 0x00ae9e18, 0x00040100, 0x00ffff28, 0x00958813, 0x00000000, 0x00000000, 0x00000000, 0x00cbb21a, 0x00fff223, 0x00000000, 0x00423b09, 0x00ffff27, 0x00ffff26, 0x00e0d01e, 0x000c0800, 0x00dec51d, 0x00ccad19, 0x00c8a919, 0x00e5c51d, 0x003d3008, 0x00ffe422, 0x00ffde20, 0x00ffe522, 0x00fedd21, 0x00423208, 0x00fbd421, 0x007c630f, 0x00a68815, 0x00f8d722, 0x00000000, 0x00000000, 0x0065360c, 0x00feb429, 
  0x00c46618, 0x005c4e0c, 0x00ffff25, 0x00917d12, 0x0068590d, 0x00080501, 0x00c9af1a, 0x00efd41f, 0x000a0701, 0x00060501, 0x00f5d420, 0x00d9bc1c, 0x0060570e, 0x00291c05, 0x0067510d, 0x00ffe925, 0x00856510, 0x00d1b91e, 0x003f3008, 0x00000000, 0x00ffe524, 0x00b48715, 0x00000000, 0x00be9317, 0x00efca21, 0x00d0aa1c, 0x00d59e1a, 0x00412c08, 0x00ffd523, 0x00a17614, 0x00ffc922, 0x00825b10, 0x00674d0d, 0x00ffc221, 0x00b97e16, 0x00ffdf26, 0x00ab7815, 0x004f370a, 0x00ffcd24, 0x005e3e0c, 0x00f3af1e, 0x00b17615, 0x00b57e17, 0x00815b10, 0x008c5011, 0x00000000, 
  0x0079400e, 0x00977913, 0x00ffff2b, 0x00ffd424, 0x00fbbb20, 0x005b410b, 0x00ffc822, 0x00ffe627, 0x00d0981a, 0x0062440c, 0x00ffdf27, 0x00ffcb24, 0x00ffe127, 0x005f410c, 0x0076540f, 0x00ffe228, 0x00ffcb24, 0x00ffe728, 0x00151003, 0x00493009, 0x00fff82d, 0x00342106, 0x00080501, 0x00ffd828, 0x00c37817, 0x00ed941d, 0x00d18b1a, 0x0057390b, 0x00ffc927, 0x00ffaf22, 0x00ffc325, 0x00402508, 0x00f99f1f, 0x00b36f16, 0x005e380b, 0x00ffed2f, 0x0074450e, 0x00b66f17, 0x00ff9b1f, 0x00362007, 0x00ffb425, 0x00ffa222, 0x00ffd02a, 0x0070480e, 0x00955212, 0x00000000, 
  0x0054320a, 0x00473809, 0x007f6212, 0x00876912, 0x004e3509, 0x001d1604, 0x00a36d14, 0x00916812, 0x004f3108, 0x002f1a04, 0x00aa7917, 0x00915810, 0x00a57315, 0x00170e03, 0x00000000, 0x00b37a18, 0x00ca8f1c, 0x00421a04, 0x00000000, 0x006b430d, 0x00a87217, 0x00000000, 0x00170c02, 0x009e6915, 0x003f2006, 0x00ac6615, 0x00351904, 0x00000000, 0x00a36014, 0x00b27216, 0x005b2c08, 0x00000000, 0x00b17117, 0x00210e02, 0x00000000, 0x008e5b11, 0x00261303, 0x00905312, 0x00522707, 0x00000000, 0x00965412, 0x00bb7f1c, 0x005e320c, 0x00381f07, 0x00faa220, 0x00000000, 
  0x00be7118, 0x004a2909, 0x003b1b06, 0x003b1d07, 0x004b300b, 0x00442507, 0x00000000, 0x00010304, 0x000a0604, 0x00211b08, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00151507, 0x00000000, 0x00000000, 0x00151809, 0x002b1b09, 0x00000000, 0x00000000, 0x001f1a09, 0x000e0a03, 0x00000000, 0x00030403, 0x00000000, 0x00080404, 0x00161106, 0x00000000, 0x000c0e07, 0x00000004, 0x00241b09, 0x00000000, 0x00000002, 0x00171309, 0x00000004, 0x00100c05, 0x00000000, 0x00080806, 0x001e1609, 0x00000000, 0x00120b01, 0x003f2d09, 0x00de871c, 0x00000000, 0x00000000, 
  0x00000000, 0x00ffb226, 0x00fea924, 0x00fea924, 0x00fec638, 0x008b4810, 0x006f6516, 0x00ffff38, 0x00ffff38, 0x00f0c92a, 0x000f0b04, 0x00f6d730, 0x008b731a, 0x000a0603, 0x00fff437, 0x0032290a, 0x00675314, 0x00ffff3e, 0x00ffe637, 0x001e1906, 0x000f0b04, 0x00ffed3a, 0x0033280a, 0x00b18623, 0x00eab72e, 0x00000000, 0x00fbc634, 0x00795d18, 0x005c4314, 0x00fff541, 0x00fff640, 0x00fac233, 0x001a1107, 0x00dba32f, 0x00ffee40, 0x00ffff47, 0x007a5d1a, 0x0036250d, 0x00ffe943, 0x00ffde41, 0x00745c1e, 0x00703e0b, 0x00ffba29, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00fe9c27, 0x004a2f09, 0x00dfdd20, 0x00b5b01c, 0x00818517, 0x00544c0c, 0x00413e0a, 0x00ffff33, 0x009f9218, 0x00817313, 0x00fee727, 0x003a3509, 0x00fff32b, 0x00ddca23, 0x00ffff34, 0x004b430c, 0x006c5c11, 0x00ffe72a, 0x00221d05, 0x00ffed2d, 0x00e8c826, 0x005a4b0f, 0x00fff430, 0x0040370b, 0x00d7b725, 0x00bfaa25, 0x0080771c, 0x005b4b10, 0x0055460f, 0x00fff334, 0x007e6118, 0x0090851e, 0x00221c06, 0x00bf9b23, 0x00d4b82c, 0x00ebc42e, 0x00dabd2b, 0x0055300a, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00be6116, 0x0049430a, 0x00ffff26, 0x00c1b719, 0x0069560c, 0x00000000, 0x009c9015, 0x00ffff2a, 0x00d2c21b, 0x00faef22, 0x00867b10, 0x006b650e, 0x00ffff25, 0x00141701, 0x0069680d, 0x00101002, 0x00ebe120, 0x008f8311, 0x003a3908, 0x00ffff2b, 0x00e8db1f, 0x00e5d820, 0x00c4b61a, 0x002f2e06, 0x00ffff29, 0x00ccc91c, 0x00746e0c, 0x00000000, 0x00a89918, 0x00ffff2b, 0x00949e17, 0x00191700, 0x00000000, 0x00ffff2b, 0x00c9b91c, 0x00eee224, 0x00696c11, 0x006f340d, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00713b0e, 0x0073640f, 0x00f2d622, 0x00dacd20, 0x00917813, 0x00000000, 0x00e7ba1e, 0x00fad221, 0x00ffce20, 0x00fff126, 0x00403308, 0x00b59817, 0x00e4c01d, 0x009b7d13, 0x006b500d, 0x00171503, 0x00ffff26, 0x00342c06, 0x00927f13, 0x00ffeb24, 0x00eed01e, 0x00ffff29, 0x0061550c, 0x0060570c, 0x00f7f723, 0x00d7de20, 0x009d9115, 0x00000100, 0x00e4d41e, 0x00d7d91e, 0x00e8e922, 0x00242205, 0x00353207, 0x00ffff27, 0x00ffff28, 0x00bbbe19, 0x00180d02, 0x00eb8a1e, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00fea924, 0x004c2d09, 0x00cd8c19, 0x00c88519, 0x0048320a, 0x00180b03, 0x00372707, 0x00ffb721, 0x00865510, 0x00ffb420, 0x00df971b, 0x003c2808, 0x00fcb520, 0x009b6c12, 0x00ffd825, 0x00946d13, 0x005c460c, 0x00ffbd21, 0x00191203, 0x00fac020, 0x00986a12, 0x00c59319, 0x00ffde25, 0x00342606, 0x00c59819, 0x00d5a81c, 0x00483f0b, 0x001b1003, 0x003b3007, 0x00fff127, 0x007a5f0f, 0x00513c0a, 0x00020000, 0x00bb9a18, 0x00d8b91d, 0x00b9a019, 0x00bea318, 0x005e320b, 0x00ffcc33, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00c06f17, 0x005b370b, 0x00ffb126, 0x00ffaf25, 0x00ffcb2b, 0x00502f0a, 0x00a76415, 0x00fc941f, 0x000b0301, 0x00ffc629, 0x00a15d14, 0x00030200, 0x00ffb726, 0x00ffab24, 0x00ffbc26, 0x00392007, 0x00ec921d, 0x00b06915, 0x005d390c, 0x00ffcb28, 0x000b0401, 0x00ee951d, 0x00fd9c1e, 0x00412808, 0x00ffc125, 0x00ffc125, 0x00ffe22b, 0x00603f0c, 0x00885b11, 0x00ffe22a, 0x00ffd227, 0x00ffb521, 0x00372507, 0x00ffca25, 0x00996112, 0x00ef9e1d, 0x00b18016, 0x005e340b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00a86514, 0x00060301, 0x0054380c, 0x004e2d0a, 0x0051310b, 0x00000000, 0x003a2208, 0x002d1906, 0x00000000, 0x004b300b, 0x00241405, 0x00130b03, 0x00482709, 0x00724d11, 0x00512b0a, 0x00000000, 0x0053300a, 0x00000000, 0x00170d03, 0x0049280a, 0x00000000, 0x00412408, 0x001e0e04, 0x00000000, 0x0054370b, 0x004d2a09, 0x0051320a, 0x00000000, 0x00251705, 0x004d330b, 0x0051380b, 0x00231304, 0x00000000, 0x0054350b, 0x00060300, 0x004d3109, 0x00080401, 0x00bb7018, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
  0x00000000, 0x00000000, 0x00000000, 0x00fb951f, 0x007b490f, 0x004c2b09, 0x004d2d09, 0x004d2b09, 0x00864f10, 0x005c370b, 0x006e410d, 0x00cc7919, 0x004d2b09, 0x00865010, 0x00ec901e, 0x004f2e09, 0x003b2007, 0x00452908, 0x00824e10, 0x0051300a, 0x0074450e, 0x0070420d, 0x004d2d09, 0x00cc7a1a, 0x006e410d, 0x005f390c, 0x007d4b0f, 0x004d2b09, 0x004d2d09, 0x004d2b09, 0x007f4b10, 0x00673c0d, 0x004d2809, 0x004d2909, 0x0061380c, 0x00834d10, 0x004d2a09, 0x0073430f, 0x00502d0a, 0x007e490f, 0x00ffa125, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000
};

// Array of all bitmaps for convenience. (Total bytes used to store images in PROGMEM = 816)
const int epd_bitmap_allArray_LEN = 1;
const unsigned long* epd_bitmap_allArray[1] = {
  logo
};
